import 'dart:io';
import 'package:args/args.dart';
import 'package:path/path.dart' as path;
import 'package:ansicolor/ansicolor.dart';

final FileSeperatorLen = 2;

List<List<String>> makeFileMatrix(List<String> files, int itemEachLine) {
  int totalLine = (files.length / itemEachLine).ceil();
  var result = new List<List<String>>.generate(totalLine, (i) => new List<String>());

  var currentLine = 0;
  for (var file in files) {
    result[currentLine].add(file);

    if (currentLine + 1 < totalLine) {
      currentLine += 1;
    } else {
      currentLine = 0;
    }
  }

  return result;
}

void printFile(String file, ArgResults result, {int columnLength}) {
  var isFolder = FileSystemEntity.isDirectorySync(file);

  var target = path.basename(file);

  if (columnLength != null) {
    target = target.padRight(columnLength);
  } else {
    target = '$target\t';
  }

  if (isFolder) {
    var pen = new AnsiPen()..xterm(031);
    target = pen(target);
  }

  stdout.write(target);
}

void printFiles(List<String> files, ArgResults result) {
  files.sort((a, b) {
    return a.toLowerCase().compareTo(b.toLowerCase());
  });
  if (!result['all']) {
    files.removeWhere((file) => path.basename(file).startsWith('.'));
  }
  final maxWidth = stdout.terminalColumns;
  int totalLine = 1;
  int itemEachLine;
  List<int> columnsLength;
  while (totalLine < files.length) {
    itemEachLine = (files.length / totalLine).ceil();
    columnsLength = new List<int>.generate(itemEachLine, (index) => 0);
    var fileMatrix = makeFileMatrix(files, itemEachLine);
    fileMatrix[0].asMap().forEach((index, value) {
      var maxLen = fileMatrix.fold(0, (target, next) {
        if (index >= next.length) {
          return target;
        }
        var fileName = path.basename(next[index]);
        if (fileName.length + FileSeperatorLen > target) {
          target = fileName.length + FileSeperatorLen;
        }
        return target;
      });
      columnsLength[index] = maxLen;
    });
    if (columnsLength.fold(0, (target, next) => target + next) < maxWidth) {
      break;
    } else {
      totalLine += 1;
    }
  }

  var fileMatrix = makeFileMatrix(files, itemEachLine);
  for (var line in fileMatrix) {
    for (var i = 0; i < line.length; i++) {
      printFile(line[i], result, columnLength: columnsLength[i]);
    }
    stdout.write('\n');
  }
}

void index(String target, ArgResults result) {
  if (FileSystemEntity.isFileSync(target)) {
    printFiles([target], result);
  } else {
    var child = new Directory(target).listSync();
    printFiles(child.map((item) => item.path).toList(), result);
  }
}